from django.utils import timezone
from django.test import TestCase
from facebook_api.signals import facebook_api_post_fetch
from facebook_pages.models import Page
from facebook_pages.factories import PageFactory
from models import PageStatistic
from datetime import datetime

PAGE_FANS_ID = 19292868552


class FacebookPageStatisticTest(TestCase):

    def test_page_statistic_create(self):

        self.assertEqual(PageStatistic.objects.count(), 0)

        page = PageFactory(graph_id=PAGE_FANS_ID, likes_count=10, talking_about_count=20)

        self.assertEqual(PageStatistic.objects.count(), 0)

        facebook_api_post_fetch.send(sender=page.__class__, instance=page, created=True)

        self.assertEqual(PageStatistic.objects.count(), 1)

        stat = page.statistics.latest()
        self.assertEqual(stat.likes_count, 10)
        self.assertEqual(stat.talking_about_count, 20)
        self.assertTrue(isinstance(stat.updated_at, datetime))

        page = Page.remote.fetch(PAGE_FANS_ID)

        self.assertEqual(PageStatistic.objects.count(), 2)

        stat = page.statistics.latest()
        self.assertTrue(stat.likes_count > 10)
        self.assertTrue(stat.talking_about_count > 20)
        self.assertTrue(isinstance(stat.updated_at, datetime))

    def test_null_stats_test(self):
        page = PageFactory(likes_count=None, talking_about_count=None)
        facebook_api_post_fetch.send(sender=page.__class__, instance=page, created=True)

        self.assertEqual(PageStatistic.objects.count(), 0)




from django.dispatch import receiver
from facebook_api.signals import facebook_api_post_fetch
from facebook_pages.models import Page
from . models import PageStatistic


@receiver(facebook_api_post_fetch, sender=Page)
def page_statistic_create(sender, instance, **kwargs):
    if instance.likes_count is None and instance.talking_about_count is None:
        return

    PageStatistic.objects.create(page=instance,
         likes_count=instance.likes_count,
         talking_about_count=instance.talking_about_count)



# -*- coding: utf-8 -*-
from django.utils import timezone
from facebook_pages.factories import PageFactory
from models import PageStatistic
import factory
import random


class PageStatisticFactory(factory.DjangoModelFactory):
    FACTORY_FOR = PageStatistic

    page = factory.SubFactory(PageFactory)
    likes_count = factory.LazyAttribute(lambda o: random.randint(0, 10000))
    talking_about_count = factory.LazyAttribute(lambda o: random.randint(0, 10000))
    updated_at = factory.LazyAttribute(lambda o: timezone.now())



from django.db import models
from facebook_pages.models import Page


class PageStatistic(models.Model):
    class Meta:
        verbose_name = "Facebook page statistic"
        verbose_name_plural = "Facebook page statistics"
        get_latest_by = 'id'

    page = models.ForeignKey(Page, related_name='statistics')

    likes_count = models.PositiveIntegerField(editable=False)
    talking_about_count = models.PositiveIntegerField(editable=False)
    updated_at = models.DateTimeField(auto_now_add=True, editable=False)

import signals



# -*- coding: utf-8 -*-
from django.contrib import admin
from .models import PageStatistic


class PageStatisticAdmin(admin.ModelAdmin):
    search_fields = ("page",)
    list_display = ("page", "likes_count", "talking_about_count", "updated_at",)


admin.site.register(PageStatistic, PageStatisticAdmin)
